<?php

namespace EasyContent\WordPress\Controllers\Actions\Main;

use EasyContent\WordPress\Controllers\Actions\BaseAction;
use EasyContent\WordPress\Implementation\Entity\DbWorkflowStatusStorage;
use EasyContent\WordPress\Implementation\Form\Defaults\SearchArticlesFormDefaults;
use EasyContent\WordPress\Implementation\Form\ImportArticlesForm;
use EasyContent\WordPress\Implementation\Form\SearchArticlesForm;
use EasyContent\WordPress\NoticesManager;
use EasyContent\WordPress\Pages\Import\ArticlesListTable;
use EasyContent\WordPress\Plugin;
use EasyContent\WordPress\Source\UseCase\RefreshCachedData\Handler as RefreshDataHandler;
use EasyContent\WordPress\Source\UseCase\Import\Handler;

class Import extends BaseAction
{
    public function run()
    {
        $plugin = Plugin::getInstance();
        $form = ImportArticlesForm::fromGlobals();

        if ($form && ($command = $form->getCommand())) {
            try {
                /** @var Handler $handler */
                $handler = $plugin->getContainer()->get(Handler::class);
                $handler->handle($command);

                $n = count($command->getArticlesIds());
                $plugin->noticesManager->addNotice("{$n} articles were added successfully!");
            } catch (\Exception $e) {
                $plugin->noticesManager->addNotice($e->getMessage(), NoticesManager::TYPE_ERROR);
            }
        }


        if (isset($_POST['action'])) {
            $action = sanitize_text_field($_POST['action']);

            if ('refresh-all' === $action) {
                /** @var RefreshDataHandler $refreshDataHandler */
                $refreshDataHandler = $plugin->getContainer()->get(RefreshDataHandler::class);
                $refreshDataHandler->handle();

                // TODO: Add success/fail messages
            }
        }

        $searchForm = new SearchArticlesForm(new SearchArticlesFormDefaults);
        $searchForm->loadFromGet();

        $table = new ArticlesListTable($searchForm);
        $table->prepare_items();

        /** @var DbWorkflowStatusStorage $statuses */
        $statuses = $plugin->getContainer()->get(DbWorkflowStatusStorage::class);

        return $this->render( 'admin/pages/import', [
            'table' => $table,
            'workflowStages' => $statuses->all(),
            'searchForm' => $searchForm,
            'statusesInUse' => $statuses->findInUse(),
        ] );
    }
}
