<?php

namespace EasyContent\WordPress\Controllers;

use EasyContent\WordPress\Controllers\Actions\BaseAction;
use EasyContent\WordPress\Controllers\Actions\Main\Connection;
use EasyContent\WordPress\Controllers\Actions\Main\ImageProcessing;
use EasyContent\WordPress\Controllers\Actions\Main\Import;
use EasyContent\WordPress\Controllers\Actions\Main\PublishingOptions;
use EasyContent\WordPress\Controllers\Actions\Main\Seo;

final class Main
{
    const ROUTES = [
        'seo'                => Seo::class,
        'import'             => Import::class,
        'connection'         => Connection::class,
        'image-processing'   => ImageProcessing::class,
        'publishing-options' => PublishingOptions::class,
    ];

    /**
     * @param string $slug
     * @return null|BaseAction
     */
    private function createAction($slug)
    {
        if (!is_string($slug) || !$slug) {
            return null;
        }

        $actionClass = array_key_exists($slug, self::ROUTES) ? self::ROUTES[$slug] : null;

        if ( ! $actionClass || ! class_exists( $actionClass ) ) {
            return null;
        }

        return new $actionClass;
    }

    /**
     * @param string $slug
     * @return string
     */
    public function renderAction($slug)
    {
        if ($action = $this->createAction($slug)) {
            return $action->run();
        }

        return '';
    }
}
