<?php

namespace EasyContent\WordPress\Db;

class BaseModel
{
    const EC_PLUGIN_DB_PREFIX = 'easycontent_';

    /**
     * @return \wpdb
     * @throws \Exception If $wpdb global is unavailable
     */
    protected static function getWpdb()
    {
        global $wpdb;

        if ( ! $wpdb ) {
            throw new \Exception( __( '$wpdb global variable was not defined', EASYCONTENT_TXTDOMAIN ) );
        }

        return $wpdb;
    }


    public static function tableName()
    {
        $wpdb = static::getWpdb();

        $reflectionClass = new \ReflectionClass( static::class );

        $className = $reflectionClass->getShortName();
        $className = preg_replace( '/([a-z])([A-Z])/', '${1}_${2}', $className );

        return $wpdb->prefix . static::EC_PLUGIN_DB_PREFIX . strtolower( $className );
    }

    public static function tableExists()
    {
        $wpdb = static::getWpdb();
        $tableName = static::tableName();
        $query = "SELECT * FROM `information_schema`.`TABLES` WHERE `TABLE_SCHEMA` = '{$wpdb->dbname}' AND `TABLE_NAME` = '{$tableName}' LIMIT 1";
        return (bool)$wpdb->get_row( $query );
    }

    public static function countAll()
    {
        $wpdb = static::getWpdb();
        $tableName = static::tableName();

        return (int)$wpdb->get_var( "SELECT COUNT(*) FROM `{$tableName}`;" );
    }


    public static function insertRow(array $data, $format = null)
    {
        $insertions = [];
        $fields = static::getFieldNames();

        foreach ( $data as $fieldName => $fieldValue ) {
            if ( in_array( $fieldName, $fields ) ) {
                $insertions[$fieldName] = $fieldValue;
            }
        }

        if ( ! $insertions ) {
            return null;
        }

        $wpdb = static::getWpdb();
        $tableName = static::tableName();

        $result = $wpdb->insert( $tableName, $insertions, $format );

        if ( false === $result ) {
            return null;
        }

        return static::create( $data );
    }


    public static function update(array $data, $where, $format = null, $whereFormat = null)
    {
        $insertions = [];
        $fields = static::getFieldNames();

        foreach ( $data as $fieldName => $fieldValue ) {
            if ( in_array( $fieldName, $fields ) ) {
                $insertions[$fieldName] = $fieldValue;
            }
        }

        if ( ! $insertions ) {
            return false;
        }

        $wpdb = static::getWpdb();
        $tableName = static::tableName();

        $result = $wpdb->update( $tableName, $insertions, $where, $format, $whereFormat );

        return false !== $result;
    }


    public static function truncate()
    {
        $wpdb = static::getWpdb();
        $tableName = static::tableName();

        $query = "TRUNCATE TABLE `{$tableName}`";

        $wpdb->query( $query );
    }


    public static function drop()
    {
        $wpdb = static::getWpdb();
        $tableName = static::tableName();

        $query = "DROP TABLE IF EXISTS `{$tableName}`";

        $wpdb->query( $query );
    }


    public static function getFieldNames()
    {
        return [];
    }


    public static function create(array $data)
    {
        $object = new static();
        $fields = static::getFieldNames();

        foreach ( $fields as $fieldName ) {
            $fieldValue = isset( $data[$fieldName] ) ? $data[$fieldName] : null;
            $object->$fieldName = $fieldValue;
        }

        return $object;
    }


    public static function deleteByPrimaryKey($value, $pk = 'id')
    {
        $wpdb = static::getWpdb();
        $tableName = static::tableName();

        $query = "DELETE FROM `{$tableName}` WHERE `{$pk}` = {$value};";

        $wpdb->query( $query );
    }


    /**
     * @param mixed $value
     * @param string $key
     * @return null|static
     * @throws \Exception
     */
    public static function findByKey($value, $key = 'id')
    {
        $wpdb = static::getWpdb();
        $tableName = static::tableName();

        $query = "SELECT * FROM `{$tableName}` WHERE `{$key}` = {$value} LIMIT 1";

        $row = $wpdb->get_row( $query, ARRAY_A );

        return $row ? static::create( $row ) : null;
    }
}
