<?php

namespace EasyContent\WordPress\Domain\Seo;

/**
 * Used to represent and modify some data from `wp_aioseo_posts` table
 */
final class SeoPackPost
{
    /** @var int $id */
    private $id;
    /** @var int $postId */
    private $postId;
    /** @var null|string $title */
    private $title;
    /** @var null|string $description */
    private $description;

    private function __construct() { }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int
     */
    public function getPostId()
    {
        return $this->postId;
    }

    /**
     * @return string|null
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param string|null $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * @return string|null
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string|null $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * @param array $payload
     * @return self
     */
    public static function fromState(array $payload)
    {
        $tmp = array_combine(array_keys($payload), array_fill(0, count($payload), true));

        if (!isset($tmp['id'], $tmp['post_id'], $tmp['title'], $tmp['description'])) {
            throw new \DomainException('Unable to create SeoPackPost from state: some parameters missing');
        }

        $model = new self();

        $model->id = $payload['id'];
        $model->postId = $payload['post_id'];
        $model->title = $payload['title'];
        $model->description = $payload['description'];

        return $model;
    }
}
