<?php

namespace EasyContent\WordPress\Helpers;

use EasyContent\WordPress\Implementation\Form\PublishingOptionsForm;
use EasyContent\WordPress\Plugin;

final class PublishingOptionsPageHelper
{
    /**
     * @param array|\WP_Term[] $list
     * @param null|integer $parentId
     * @param string $idAttrName
     * @param string $parentAttrName
     *
     * @return \stdClass[]|\WP_Term[] Sorted array of objects
     */
    public static function createCategoriesDataStructure(
        array $list,
        $parentId = null,
        $idAttrName = 'term_id',
        $parentAttrName = 'parent'
    ) {
        $collection = [];
        $parentId = $parentId ?: 0;

        /** @var array|\WP_Term[] $list */
        foreach ( $list as $i => $category ) {
            $tmpCat = is_object( $category ) ? $category : (object)$category;

            if ( $parentId != $tmpCat->{$parentAttrName} ) {
                continue;
            }

            $tmpCat->children = self::createCategoriesDataStructure( $list, $tmpCat->{$idAttrName}, $idAttrName, $parentAttrName );
            $collection[] = $tmpCat;
        }

        return $collection;
    }

    public static function renderCategoriesList(array $list, $name, $id, $selected = '', $emptyOption = false, $idAttrName = 'term_id')
    {
        $layout = sprintf( '<select class="regular-text" name="%s" id="%s">' . PHP_EOL, esc_attr( $name ), esc_attr( $id ) );

        if ( $emptyOption ) {
            $layout .= '<option value="" ' . selected( $selected, '', false ) . '></option>';
        }

        $layout .= self::renderCategoriesListOptions( $list, $selected, $idAttrName );
        $layout .= '</select>' . PHP_EOL;

        return $layout;
    }

    private static function renderCategoriesListOptions(array $list, $selected, $idAttrName = 'term_id', $level = 0)
    {
        $layout = '';

        foreach ( $list as $i => $listItem ) {
            $title = str_repeat( '&mdash; ', $level ) . $listItem->name;
            $s = selected( $selected, $listItem->{$idAttrName}, false );

            $layout .= sprintf( '<option value="%s" %s>%s</option>', esc_attr( $listItem->{$idAttrName} ), $s, esc_html( $title ) ) . PHP_EOL;

            if ( $listItem->children ) {
                $layout .= self::renderCategoriesListOptions( $listItem->children, $selected, $idAttrName, $level + 1 );
            }
        }

        return $layout;
    }

    public static function renderCategoriesTableBody(array $categoriesList, array $ecCatList, $level = 0)
    {
        $layout = '';

        foreach ( $categoriesList as $i => $listItem ) {
            $layout .= self::renderCategoriesTableBodyRow( $listItem, $ecCatList, $level );

            if ( $listItem->children ) {
                $layout .= self::renderCategoriesTableBody( $listItem->children, $ecCatList, $level + 1 );
            }
        }

        return $layout;
    }

    private static function renderCategoriesTableBodyRow($listItem, $ecCatList, $level = 0)
    {
        $title = str_repeat( '&mdash; ', $level ) . $listItem->name;
        $title = esc_html( $title );

        $selectId   = "ec-category-{$listItem->term_id}";
        $selectName = sprintf('%s[%s][%s]', PublishingOptionsForm::NAME, PublishingOptionsForm::FIELD_CATEGORY_MAPPINGS, $listItem->term_id);

        $mappings = Plugin::getInstance()->getSettings()->mappings;
        $selected = ! empty( $mappings[$listItem->term_id] ) ? (int)$mappings[$listItem->term_id] : 0;

        $selectHtml = self::renderCategoriesList( $ecCatList, $selectName, $selectId, $selected, true, 'id' );

        return "<tr><td>{$title}</td><td>{$selectHtml}</td></tr>";
    }
}
