<?php

namespace EasyContent\WordPress\Implementation;

final class Container
{
    private $config;
    private $instances = [];

    public function __construct(array $config)
    {
        $this->config = $config;
    }

    /**
     * @param string $id
     * @return mixed
     * @throws \Exception
     */
    public function get($id)
    {
        if (!$this->has($id)) {
            throw new \Exception("Can't get '{$id}' instance"); // TODO: NotFoundExceptionInterface
        }

        return $this->buildInstance($id);
    }

    /**
     * @param string $id
     * @return bool
     */
    public function has($id)
    {
        return isset($this->config[$id]);
    }

    /**
     * @param string $className
     * @param null|string $parent
     * @return mixed
     * @throws \Exception
     */
    private function buildInstance($className, $parent = null)
    {
        if (isset($this->instances[$className])) {
            return $this->instances[$className];
        }

        if (!class_exists($className)) {
            throw new \Exception("Class '{$className}' doesn't exist"); // TODO: ContainerExceptionInterface
        }

        if (!$this->has($className)) {
            return new $className;
        }

        $dependencies = $this->config[$className];

        if (!$dependencies) {
            return new $className;
        }

        if ($parent && isset($dependencies[$parent])) {
            throw new \Exception("Can't resolve circular dependencies. {$className} <-> {$parent}"); // TODO: ContainerExceptionInterface
        }

        $builtDependencies = [];

        foreach ($dependencies as $i => $dependency) {
            if (is_callable($dependency)) {
                $builtDependencies[] = $dependency();
            } elseif (is_string($dependency) && class_exists($dependency)) {
                $builtDependencies[] = $this->buildInstance($dependency, $className);
            } else {
                throw new \Exception("Can't build '{$i}' dependency for '{$className}'"); // TODO: ContainerExceptionInterface
            }
        }

        $instance = new $className(...$builtDependencies);
        $this->instances[$className] = $instance;

        return $instance;
    }
}
