<?php

namespace EasyContent\WordPress\Implementation\Db\Migrations;

use EasyContent\WordPress\Implementation\Db\Migration;

final class InitialMigration implements Migration
{
    public function run()
    {
        global $wpdb;

        if (!function_exists('dbDelta')) {
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        }

        $charsetCollate = $wpdb->get_charset_collate();

        $articlesTableName = "{$wpdb->prefix}easycontent_article";
        $workflowStatusesTableName = "{$wpdb->prefix}easycontent_workflow_stage";
        $categoriesTableName = "{$wpdb->prefix}easycontent_category";
        $articlePostRelationTableName = "{$wpdb->prefix}easycontent_article_post";

        $queries = [
            "CREATE TABLE `{$articlesTableName}` (
                `id` INT UNSIGNED NOT NULL PRIMARY KEY,
                `current_workflow_stage_id` INT UNSIGNED NOT NULL,
                `category_id` INT UNSIGNED NULL DEFAULT 0,
                `topic_name` VARCHAR(255) NOT NULL,
                `title` VARCHAR(255) NULL DEFAULT NULL,
                `updated` INT UNSIGNED NULL DEFAULT 0
            ) {$charsetCollate};",
            "CREATE TABLE `{$workflowStatusesTableName}` (
                `id` INT UNSIGNED NOT NULL PRIMARY KEY,
                `color` VARCHAR(6) NOT NULL,
                `name` VARCHAR(255) NOT NULL,
                `is_core` TINYINT UNSIGNED NOT NULL DEFAULT 0
            ) {$charsetCollate};",
            "CREATE TABLE `{$categoriesTableName}` (
                `id` INT UNSIGNED NOT NULL PRIMARY KEY,
                `parent_category_id` INT UNSIGNED NOT NULL DEFAULT 0,
                `name` VARCHAR(255) NOT NULL
            ) {$charsetCollate};",
            "CREATE TABLE `{$articlePostRelationTableName}` (
                `id` INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
                `article_id` INT UNSIGNED NOT NULL,
                `post_id` INT UNSIGNED NOT NULL,
                `sync_time` INT UNSIGNED NOT NULL,
                UNIQUE INDEX `article_id-post_id-idx` (`article_id`, `post_id`)
            ) {$charsetCollate};",
        ];

        dbDelta($queries);
    }
}
