<?php

namespace EasyContent\WordPress\Implementation\Entity\Db;

use InvalidArgumentException;

final class Version
{
    // Default version value
    const DEF = '0.0.0';

    private $value;

    public function __construct($version)
    {
        if (!is_string($version)) {
            throw new InvalidArgumentException('Version must be string');
        }

        if (!self::test($version)) {
            $message = sprintf('"%s" is invalid version format', $version);
            throw new InvalidArgumentException($message);
        }

        $this->value = $version;
    }

    /**
     * @return Version
     */
    public static function initial()
    {
        return new self(self::DEF);
    }

    /**
     * @return string
     */
    public function asString()
    {
        return $this->value;
    }

    /**
     * @param Version $version
     * @return bool
     */
    public function equals($version)
    {
        return version_compare($this->value, $version->value) === 0;
    }

    /**
     * @param Version $version
     * @return bool
     */
    public function notEquals($version)
    {
        return !$this->equals($version);
    }

    /**
     * @param Version $version
     * @return bool
     */
    public function greaterThan($version)
    {
        return version_compare($this->value, $version->value, '>');
    }

    /**
     * @return bool
     */
    public function isDefault()
    {
        return version_compare($this->value, self::DEF) === 0;
    }

    /**
     * @param string $version
     * @return bool Returns TRUE if $version string has valid format and FALSE in other case
     */
    public static function test($version)
    {
        $positiveIntegerPattern = '[1-9]\d*';
        $versionPartPattern = "({$positiveIntegerPattern})|0";
        $versionPattern = "/^({$versionPartPattern})\.({$versionPartPattern})\.({$versionPartPattern})$/m";

        return 1 === preg_match($versionPattern, $version);
    }
}
