<?php

namespace EasyContent\WordPress\Implementation\Entity;

use EasyContent\WordPress\Source\Api\Entity\Category;
use EasyContent\WordPress\Source\Entity\CategoriesStorage;

final class DbCategoriesStorage implements CategoriesStorage
{
    private $tableName;

    public function __construct()
    {
        global $wpdb;
        $this->tableName = "{$wpdb->prefix}easycontent_category";
    }

    /**
     * @return array|Category[]
     */
    public function all()
    {
        global $wpdb;
        $query = "SELECT * FROM `{$this->tableName}`;";
        $rows = $wpdb->get_results($query, ARRAY_A);

        if (!is_array($rows)) {
            return [];
        }

        return array_map([$this, 'fromRow'], $rows);
    }

    /**
     * @param array|Category[] $items
     * @return void
     */
    public function batchDelete(array $items)
    {
        if (!$items) {
            return;
        }

        global $wpdb;
        $ids = array_map(function(Category $c) { return $c->getId(); }, $items);
        $chunks = array_chunk($ids, 500);

        foreach ($chunks as $chunk) {
            $idsStr = implode(', ', $chunk);
            $query = "DELETE FROM `{$this->tableName}` WHERE `id` IN ({$idsStr});";
            $wpdb->query($query);
        }
    }

    /**
     * @param Category $item
     * @return void
     */
    public function insert($item)
    {
        global $wpdb;
        $wpdb->insert(
            $this->tableName,
            $this->toArray($item),
            ['%d', '%d', '%s']
        );
    }

    /**
     * @param Category $item
     * @return void
     */
    public function update($item)
    {
        global $wpdb;
        $wpdb->update(
            $this->tableName,
            $this->toArray($item),
            ['id' => $item->getId()],
            ['%d', '%d', '%s'],
            ['%d']
        );
    }

    /**
     * Maps from DB value to PHP value
     *
     * @param array $row
     * @return Category
     */
    private function fromRow(array $row)
    {
        return Category::fromState([
            'id' => (int)$row['id'],
            'parent_category_id' => (int)$row['parent_category_id'],
            'name' => $row['name'],
        ]);
    }

    /**
     * Maps from PHP value to DB value
     *
     * @param Category $category
     * @return array
     */
    private function toArray(Category $category)
    {
        return [
            'id' => $category->getId(),
            'parent_category_id' => $category->getParentCategoryId(),
            'name' => $category->getName(),
        ];
    }
}
