<?php

namespace EasyContent\WordPress\Implementation\Entity;

use EasyContent\WordPress\Source\Api\Entity\WorkflowStatus;
use EasyContent\WordPress\Source\Entity\WorkflowStatusStorage;
use EasyContent\WordPress\Source\Exception\WorkflowStatusNotFoundException;

final class DbWorkflowStatusStorage implements WorkflowStatusStorage
{
    private $tableName;

    public function __construct()
    {
        global $wpdb;
        $this->tableName = "{$wpdb->prefix}easycontent_workflow_stage";
    }

    public function existsById($id)
    {
        global $wpdb;
        $query = "SELECT `id` FROM `{$this->tableName}` WHERE `id` = %d LIMIT 1;";
        $query = $wpdb->prepare($query, $id);

        return $wpdb->get_var($query) !== null;
    }

    /**
     * @return array|WorkflowStatus[]
     */
    public function all()
    {
        global $wpdb;
        $query = "SELECT * FROM `{$this->tableName}`;";
        $rows = $wpdb->get_results($query, ARRAY_A);

        if (!is_array($rows)) {
            return [];
        }

        return array_map([$this, 'fromRow'], $rows);
    }

    /**
     * @param array|WorkflowStatus[] $items
     * @return void
     */
    public function batchDelete(array $items)
    {
        if (!$items) {
            return;
        }

        global $wpdb;
        $ids = array_map(function(WorkflowStatus $s) { return $s->getId(); }, $items);
        $chunks = array_chunk($ids, 500);

        foreach ($chunks as $chunk) {
            $idsStr = implode(', ', $chunk);
            $query = "DELETE FROM `{$this->tableName}` WHERE `id` IN ({$idsStr});";
            $wpdb->query($query);
        }
    }

    /**
     * @param WorkflowStatus $item
     * @return void
     */
    public function insert($item)
    {
        global $wpdb;
        $wpdb->insert(
            $this->tableName,
            $this->toArray($item),
            ['%d', '%s', '%s', '%d']
        );
    }

    /**
     * @param WorkflowStatus $item
     * @return void
     */
    public function update($item)
    {
        global $wpdb;
        $wpdb->update(
            $this->tableName,
            $this->toArray($item),
            ['id' => $item->getId()],
            ['%d', '%s', '%s', '%d'],
            ['%d']
        );
    }

    /**
     * Maps from DB value to PHP value
     *
     * @param array $row
     * @return WorkflowStatus
     */
    private function fromRow(array $row)
    {
        return WorkflowStatus::fromState([
            'id' => (int)$row['id'],
            'color' => $row['color'],
            'name' => $row['name'],
            'is_core' => (int)$row['is_core'],
        ]);
    }

    /**
     * Maps from PHP value to DB value
     *
     * @param WorkflowStatus $status
     * @return array
     */
    private function toArray(WorkflowStatus $status)
    {
        return [
            'id' => $status->getId(),
            'color' => $status->getColor(),
            'name' => $status->getName(),
            'is_core' => $status->getIsCore(),
        ];
    }

    public function getDraft()
    {
        global $wpdb;
        $query = "SELECT * FROM `{$this->tableName}` WHERE `is_core` = 1 LIMIT 1;";
        $row = $wpdb->get_row($query, ARRAY_A);

        if ($row === null) {
            throw new \Exception('Draft status missing. Please update cache in order to fix this issue.');
        }

        return self::fromRow($row);
    }

    public function getById($id)
    {
        global $wpdb;
        $query = "SELECT * FROM `{$this->tableName}` WHERE `id` = %d LIMIT 1;";
        $query = $wpdb->prepare($query, $id);
        $row = $wpdb->get_row($query, ARRAY_A);

        if (null === $row) {
            throw new WorkflowStatusNotFoundException;
        }

        return $this->fromRow($row);
    }

    public function findInUse()
    {
        global $wpdb;
        $articlesTableName = "{$wpdb->prefix}easycontent_article";

        $query = "SELECT `{$this->tableName}`.*
            FROM `{$this->tableName}`
            RIGHT JOIN `{$articlesTableName}` ON `{$articlesTableName}`.`current_workflow_stage_id` = `{$this->tableName}`.`id`
            GROUP BY `{$this->tableName}`.`id`;";

        $result = $wpdb->get_results($query, ARRAY_A);

        return null === $result ? [] : array_map([$this, 'fromRow'], $result);
    }
}
