<?php

namespace EasyContent\WordPress\Implementation\Entity;

use EasyContent\WordPress\Implementation\Http\Client;
use EasyContent\WordPress\Implementation\Http\Method;
use EasyContent\WordPress\Plugin;
use EasyContent\WordPress\Source\Api\Entity\Category;
use EasyContent\WordPress\Source\Api\Entity\ImportedArticle;
use EasyContent\WordPress\Source\Api\Entity\Responses\ConnectionResponse;
use EasyContent\WordPress\Source\Api\Entity\WorkflowStatus;
use EasyContent\WordPress\Source\Entity\EasyContentStorage;

final class RemoteEasyContentStorage implements EasyContentStorage
{
    public function fetchAll($key)
    {
        $client = new Client($key);
        $response = $client->request(Method::post(), 'connect');
        $payload = json_decode($response->get_data(), true);
        return ConnectionResponse::fromState($payload);
    }

    public function getCategories()
    {
        $response = $this->getClient()->request(Method::get(), 'categories');
        $payload = json_decode($response->get_data(), true);

        return array_map(function(array $row) { return Category::fromState($row); }, $payload);
    }

    public function getArticles()
    {
        $response = $this->getClient()->request(Method::get(), 'articles');
        $payload = json_decode($response->get_data(), true);

        // This code is used to fetch articles from EasyContent.io, convert them to cached articles and update DB cache.
        // No need to fetch cached data or related post from DB. However, if this method will be used anywhere else
        // we need to be sure everything works fine this way, or we need to add this data to each imported article.
        $stub = ['cached' => null, 'related_post' => null,];

        return array_map(function(array $row) use ($stub) {
            return ImportedArticle::fromState(
                array_merge($row, $stub)
            );
        }, $payload);
    }

    public function getStatuses()
    {
        $response = $this->getClient()->request(Method::get(), 'stages');
        $payload = json_decode($response->get_data(), true);

        return array_map(function(array $row) { return WorkflowStatus::fromState($row); }, $payload);
    }

    private function getClient()
    {
        return Client::fromPluginSettings();
    }
}
