<?php

namespace EasyContent\WordPress\Implementation\Form;

use EasyContent\WordPress\Source\UseCase\Import\Command;

final class ImportArticlesForm
{
    const NAME = 'ECImportArticlesForm';

    const FIELD_NEW_STAGE = 'newStage';
    const FIELD_TOPIC_IDS = 'topic-ids';

    private $newStageId;
    private $articleIds;

    private function __construct() { }

    private function setNewStage($location)
    {
        $this->newStageId = !empty($_POST[self::NAME][$location][self::FIELD_NEW_STAGE])
            ? absint($_POST[self::NAME][$location][self::FIELD_NEW_STAGE])
            : null;

        return $this;
    }

    private function setArticleIds()
    {
        $articleIds = isset($_POST[self::NAME][self::FIELD_TOPIC_IDS])
            ? array_map('absint', (array)$_POST[self::NAME][self::FIELD_TOPIC_IDS])
            : [];
        $this->articleIds = array_unique($articleIds, SORT_NUMERIC);

        return $this;
    }

    /**
     * @return Command|null
     */
    public function getCommand()
    {
        if (!$this->articleIds) {
            return null;
        }

        return new Command($this->articleIds, $this->newStageId);
    }

    /**
     * @return null|self
     */
    public static function fromGlobals()
    {
        $location = isset($_POST[self::NAME]['run'])
            ? sanitize_text_field($_POST[self::NAME]['run'])
            : null;

        if (!$location) {
            return null;
        }

        return (new self)
            ->setNewStage($location)
            ->setArticleIds();
    }
}
