<?php

namespace EasyContent\WordPress;

use EasyContent\WordPress\ValueObjects\AttachmentPost;

final class MediaList
{
    /** @var array|AttachmentPost[] $mediaFiles */
    private $mediaFiles;

    /** @var array|int[] $sourceUrls */
    private $sourceUrls;

    /**
     * @return array|AttachmentPost[]
     */
    public function getMediaFiles()
    {
        if (null !== $this->mediaFiles) {
            return $this->mediaFiles;
        }

        global $wpdb;

        $query = "SELECT `ID`, `meta_value`
            FROM `{$wpdb->posts}`
            LEFT JOIN `{$wpdb->postmeta}` ON `{$wpdb->postmeta}`.`post_id` = `ID` AND `{$wpdb->postmeta}`.`meta_key` = '_wp_attached_file'
            WHERE `post_type` = 'attachment';";

        $this->mediaFiles = [];
        $rows = $wpdb->get_results($query, ARRAY_A);

        if (null === $rows) {
            return $this->mediaFiles;
        }

        foreach ($rows as $row) {
            if (!$post = AttachmentPost::fromArray($row)) {
                continue;
            }

            $this->mediaFiles[$row['ID']] = $post;
        }

        return $this->mediaFiles;
    }

    public function getSourceUrls()
    {
        if ( null !== $this->sourceUrls ) {
            return $this->sourceUrls;
        }

        global $wpdb;

        $query = "SELECT `post_id`, `meta_value`
            FROM `{$wpdb->postmeta}`
            WHERE `meta_key` = 'ec-source-url';";

        $rows = $wpdb->get_results( $query, ARRAY_A );

        if ( null === $rows ) {
            return $this->sourceUrls = [];
        }

        $ids  = array_column( $rows, 'post_id' );
        $urls = array_column( $rows, 'meta_value' );

        return $this->sourceUrls = array_combine( $urls, $ids );
    }
}
