<?php

namespace EasyContent\WordPress\Metaboxes;

use EasyContent\WordPress\Implementation\Formatters\StatusesListFormatter;
use EasyContent\WordPress\Post;
use EasyContent\WordPress\Source\Entity\WorkflowStatusStorage;
use EasyContent\WordPress\View;

final class PostSyncMetaBox
{
    private $id;
    private $title;
    private $screen;
    private $context;
    private $priority;
    private $callbackArguments;

    private $statuses;

    public function __construct(WorkflowStatusStorage $statuses)
    {
        $this->statuses = $statuses;

        $this->id = 'ec-post-synchronization-metabox';
        $this->title = __( 'EasyContent', EASYCONTENT_TXTDOMAIN ) . ' <span class="dashicons dashicons-randomize"></span>';
        $this->screen = ['post', 'page'];
        $this->context = 'side';
        $this->priority = 'high';
        $this->callbackArguments = [
            '__back_compat_meta_box' => true
        ];

        add_meta_box( $this->id, $this->title, [$this, 'render'], $this->screen, $this->context, $this->priority, $this->callbackArguments );
    }


    public function render($post, $params)
    {
        try {
            $p = new Post( $post );
            $article = $p->getArticle();
            $stages = $p->isSynced()
                ? (new StatusesListFormatter($this->statuses->all()))->asArray()
                : [];

            echo ( new View() )->render( 'admin/metaboxes/post-sync', [
                'post' => $p,
                'stages' => $stages,
                'currentStage' => $article ? $article->getCurrentWorkflowStage() : null
            ] );
        } catch ( \Exception $e ) {
        }
    }
}
