<?php

namespace EasyContent\WordPress\Notices;

final class AdminNoticesService
{
    /** @var array|AdminNoticeInterface[] $notices */
    private $notices = [
        AdminNoticeInterface::TYPE_SUCCESS => [],
        AdminNoticeInterface::TYPE_ERROR   => [],
        AdminNoticeInterface::TYPE_INFO    => [],
        AdminNoticeInterface::TYPE_WARNING => []
    ];

    public function __construct()
    {
        add_action( 'admin_notices', function() {
            echo $this->getNoticesLayout();
        } );
    }

    public function addAdminNotice(AdminNoticeInterface $notice)
    {
        $this->notices[$notice->getType()][] = $notice;
    }

    private function getNoticesLayout()
    {
        $layout = '';

        foreach ( $this->notices as $type => $notices ) {
            if ( ! $this->notices[$type] ) {
                continue;
            }

            /** @var AdminNoticeInterface $notice */
            foreach ( $notices as $notice ) {
                $layout .= '<div class="' . $this->getNoticeClass( $notice ) . '"><p>' . $notice->getMessage() . '</p></div>';
            }
        }

        return $layout;
    }

    private function getNoticeClass(AdminNoticeInterface $notice)
    {
        $classes = ['notice', 'notice-' . $notice->getType()];

        if ( $notice->isDismissible() ) {
            $classes[] = 'is-dismissible';
        }

        return esc_attr( implode( ' ', $classes ) );
    }
}
