<?php

namespace EasyContent\WordPress\Pages\Import;

use EasyContent\WordPress\Db\Models\Article;
use EasyContent\WordPress\Helpers\TimeHelper;
use EasyContent\WordPress\Implementation\Form\SearchArticlesForm;
use EasyContent\WordPress\WP_List_Table;

final class ArticlesListTable extends WP_List_Table
{
    private $form;

    public function __construct(SearchArticlesForm $form = null, $args = [])
    {
        parent::__construct($args);
        $this->form = $form;
    }

    public function prepare_items()
    {
        $this->_column_headers = [$this->get_columns(), [], $this->get_sortable_columns()];

        $perPage = $this->get_items_per_page( 'posts_per_page' );

        $this->set_pagination_args( [
            'total_items' => Article::countListTableData($this->form),
            'per_page'    => $perPage
        ] );

        $this->items = $this->form
            ? Article::getListTableData($this->form, $this->get_pagenum(), $perPage)
            : [];
    }

    public function get_columns()
    {
        return [
            'cb' => __( 'ID', EASYCONTENT_TXTDOMAIN ),
            'topic_name' => __( 'EC Topic Name', EASYCONTENT_TXTDOMAIN ),
            'title' => __( 'EC Title', EASYCONTENT_TXTDOMAIN ),
            'updated' => __( 'Updated', EASYCONTENT_TXTDOMAIN ),
            'status' => __( 'EC Status', EASYCONTENT_TXTDOMAIN ),
            'last_synced' => __( 'Last Synced', EASYCONTENT_TXTDOMAIN )
        ];
    }

    protected function get_sortable_columns()
    {
        return [
            'topic_name' => ['topic_name', false],
            'title' => ['title', false],
            'updated' => ['updated', false],
            'status' => ['status', false],
            'last_synced' => ['last_synced', false]
        ];
    }

    protected function column_cb($item)
    {
        return sprintf( '<input type="checkbox" name="ECImportArticlesForm[topic-ids][]" value="%s" />', $item['id'] );
    }

    protected function column_topic_name($item)
    {
        return $item['topic_name'];
    }

    protected function column_title($item)
    {
        return $item['title'] ?: '&mdash;';
    }

    protected function column_updated($item)
    {
        return $item['updated']
            ? TimeHelper::getArticleListTableDateString( $item['updated'] )
            : __( 'Never', EASYCONTENT_TXTDOMAIN );
    }

    protected function column_status($item)
    {
        // TODO: move styles
        return '<span style="border-radius: 50%; background-color: #' . $item['color'] . '; width: 20px; height: 20px; display: inline-block">&nbsp;</span> '
            . $item['name'];
    }

    protected function column_last_synced($item)
    {
        if ( ! $item['sync_time'] ) {
            return __( 'Never', EASYCONTENT_TXTDOMAIN );
        }

        $editPostUrl = get_edit_post_link( $item['post_id'], '' );
        $rawDate = TimeHelper::timestampToDate( $item['sync_time'] );
        $date = TimeHelper::getArticleListTableDateString( $item['sync_time'] );

        if ( null === $editPostUrl ) {
            return sprintf( '<span title="%s">%s</span>', $rawDate, $date );
        }

        return sprintf( '<a href="%s" target="_blank" title="%s">%s</a>', $editPostUrl, $rawDate, $date );
    }
}
