<?php

namespace EasyContent\WordPress\Rest\Actions;

use EasyContent\WordPress\Plugin;
use EasyContent\WordPress\Source\Services\FrontendDataProvider;
use EasyContent\WordPress\Source\UseCase\PushPost\Command as PushPostCommand;
use EasyContent\WordPress\Source\UseCase\PushPost\Handler as PushPostHandler;

class PushPost extends BaseAction
{
    protected $methods = ['POST'];


    public function getArgumentsConfig()
    {
        return [
            'post_id' => [
                'type'              => 'integer',
                'required'          => true,
                'sanitize_callback' => 'absint'
            ]
        ];
    }


    public function run(\WP_REST_Request $request)
    {
        try {
            if ( ! Plugin::getInstance()->getSettings()->isPluginConnected ) {
                $message = __( "EasyContent plugin isn't connected to your EasyContent account. Please connect it on the EasyContent connection page in your plugin menu", EASYCONTENT_TXTDOMAIN );
                throw new \Exception( $message, 11 );
            }

            $postId = (int)$request['post_id'];
            $container = Plugin::getInstance()->getContainer();

            $command = new PushPostCommand($postId);
            /** @var PushPostHandler $handler */
            $handler = $container->get(PushPostHandler::class);

            $handler->handle($command);

            /** @var FrontendDataProvider $dataProvider */
            $dataProvider = $container->get(FrontendDataProvider::class);

            return new \WP_REST_Response($dataProvider->getData($postId));
        } catch ( \Exception $e ) {
            return new \WP_Error( $e->getCode(), $e->getMessage() );
        }
    }
}
