<?php

namespace EasyContent\WordPress\Rest\Actions;

use EasyContent\WordPress\Plugin;
use EasyContent\WordPress\Source\UseCase\SetNewStatus\Command;
use EasyContent\WordPress\Source\UseCase\SetNewStatus\Handler;

class SetArticleWorkflowStage extends BaseAction
{
    protected $methods = ['PATCH'];

    public function getArgumentsConfig()
    {
        return [
            'post_id' => [
                'type'              => 'integer',
                'required'          => true,
                'sanitize_callback' => 'absint'
            ]
        ];
    }

    public function run(\WP_REST_Request $request)
    {
        try {
            $body = (string)$request->get_body();
            $body = '' === $body ? [] : json_decode( $body, true );

            $postId = isset($request['post_id']) ? (int)$request['post_id'] : 0;
            $workflowStageId = isset( $body['workflow_stage_id'] ) ? (int)$body['workflow_stage_id'] : 0;

            $command = new Command($postId, $workflowStageId);
            /** @var Handler $handler */
            $handler = Plugin::getInstance()->getContainer()->get(Handler::class);

            $handler->handle($command);

            return new \WP_REST_Response( null, 204 );
        } catch (\Exception $e) {
            return new \WP_Error( $e->getCode(), $e->getMessage() );
        }
    }
}
