<?php

namespace EasyContent\WordPress\Rest;

use EasyContent\WordPress\Implementation\Container;
use EasyContent\WordPress\Rest\Actions\BaseAction;

final class RestManager
{
    const EASYCONTENT_V1_NAMESPACE = 'easycontent/v1';

    /** @var string $namespace */
    private $namespace;
    /** @var array<string, string> $actionsConfig */
    private $actionsConfig;
    /** @var Container $container */
    private $container;

    /**
     * @param string $namespace
     * @param array<string, string> $config
     * @param Container $container
     */
    public function __construct($namespace, array $config, Container $container)
    {
        $this->namespace = $namespace;
        $this->actionsConfig = $config;
        $this->container = $container;

        add_action('rest_api_init', [$this, 'registerRoutes']);
    }

    public function registerRoutes()
    {
        foreach ( $this->actionsConfig as $route => $className ) {
            /** @var BaseAction $action */
            $action = $this->container->get($className);

            register_rest_route($this->namespace, $route, [
                'methods'  => $action->getMethodsString(),
                'callback' => [$action, 'run'],
                'permission_callback' => [$action, 'checkAccess'],
                'args' => $action->getArgumentsConfig()
            ]);
        }
    }
}
