<?php

namespace EasyContent\WordPress\Source\Api\Entity;

abstract class BaseArticle extends Entity
{
    /** @var null|int $id */
    protected $id;
    /** @var int $currentWorkflowStatusId */
    protected $currentWorkflowStatusId;
    /** @var string $topicName */
    protected $topicName;
    /** @var null|int $updated */
    protected $updated;
    /** @var array|int[] $categoriesIds */
    protected $categoriesIds = [];
    /** @var string $title */
    protected $title;
    /** @var string $metaTitle */
    protected $metaTitle;
    /** @var string $metaDescription */
    protected $metaDescription;

    final protected function __construct() { }

    /**
     * @param array $data
     * @return static
     */
    public static function fromState(array $data)
    {
        return (new static)
            ->setId($data)
            ->setCurrentWorkflowStatusIdFromPayload($data)
            ->setTopicName($data)
            ->setUpdated($data)
            ->setCategoriesIds($data)
            ->setTitle($data)
            ->setMetaTitle($data)
            ->setMetaDescription($data);
    }

    /**
     * @return null|int
     */
    final public function getId()
    {
        return $this->id;
    }

    /**
     * @param array $data
     * @return static
     */
    private function setId(array $data)
    {
        $this->ensureArrayKeyExists('id', $data);
        $this->ensureInteger($data['id']);
        $this->id = $data['id'];
        return $this;
    }

    /**
     * @return int
     */
    final public function getCurrentWorkflowStatusId()
    {
        return $this->currentWorkflowStatusId;
    }

    /**
     * @param array $data
     * @return static
     */
    private function setCurrentWorkflowStatusIdFromPayload(array $data)
    {
        $key = 'current_workflow_stage_id';
        $this->ensureArrayKeyExists($key, $data);
        $this->ensureInteger($data[$key]);
        $this->currentWorkflowStatusId = $data[$key];
        return $this;
    }

    /**
     * @return string
     */
    final public function getTopicName()
    {
        return $this->topicName;
    }

    /**
     * @param array $data
     * @return static
     */
    private function setTopicName(array $data)
    {
        $this->ensureArrayKeyExists('topic_name', $data);
        $this->ensureString($data['topic_name']);
        $this->topicName = $data['topic_name'];
        return $this;
    }

    /**
     * @return int
     */
    final public function getUpdated()
    {
        return $this->updated;
    }

    /**
     * @param array $data
     * @return static
     */
    private function setUpdated(array $data)
    {
        $this->ensureArrayKeyExists('updated', $data);

        if ($data['updated'] === null) {
            return $this;
        }

        $this->ensureInteger($data['updated']);
        $this->updated = $data['updated'];
        return $this;
    }

    /**
     * @return array|int[]
     */
    final public function getCategoriesIds()
    {
        return $this->categoriesIds;
    }

    /**
     * @param array $data
     * @return static
     */
    private function setCategoriesIds(array $data)
    {
        $this->ensureArrayKeyExists('categories', $data);
        $this->ensureArray($data['categories']);

        foreach ($data['categories'] as $categoryId) {
            $this->ensureInteger($categoryId);
            $this->categoriesIds[] = $categoryId;
        }

        return $this;
    }

    /**
     * @return string
     */
    final public function getTitle()
    {
        return $this->title;
    }

    /**
     * @param array $data
     * @return static
     */
    private function setTitle(array $data)
    {
        $this->ensureArrayKeyExists('title', $data);
        $this->ensureString($data['title']);
        $this->title = $data['title'];
        return $this;
    }

    /**
     * @return string
     */
    final public function getMetaTitle()
    {
        return $this->metaTitle;
    }

    /**
     * @param array $data
     * @return static
     */
    private function setMetaTitle(array $data)
    {
        $this->ensureArrayKeyExists('meta_title', $data);
        $this->ensureString($data['meta_title']);
        $this->metaTitle = $data['meta_title'];
        return $this;
    }

    /**
     * @return string
     */
    final public function getMetaDescription()
    {
        return $this->metaDescription;
    }

    /**
     * @param array $data
     * @return static
     */
    private function setMetaDescription(array $data)
    {
        $this->ensureArrayKeyExists('meta_description', $data);
        $this->ensureString($data['meta_description']);
        $this->metaDescription = $data['meta_description'];
        return $this;
    }
}
