<?php

namespace EasyContent\WordPress\Source\Api\Entity\Responses;

use EasyContent\WordPress\Source\Api\Entity\Article;
use EasyContent\WordPress\Source\Api\Entity\Category;
use EasyContent\WordPress\Source\Api\Entity\Entity;
use EasyContent\WordPress\Source\Api\Entity\WorkflowStatus;

final class ConnectionResponse extends Entity
{
    /** @var string $accountName */
    private $accountName;
    /** @var string $projectName */
    private $projectName;
    /** @var array|Article[] $articles */
    private $articles = [];
    /** @var array|Category[] $categories */
    private $categories = [];
    /** @var WorkflowStatus[] $stages */
    private $stages = [];
    /** @var string $editUrlBase */
    private $editUrlBase;

    private function __construct() { }

    /**
     * @param array $data
     * @return self
     */
    public static function fromState(array $data)
    {
        return (new self)
            ->setAccountName($data)
            ->setProjectName($data)
            ->setArticles($data)
            ->setCategories($data)
            ->setStages($data)
            ->setEditUrlBase($data);
    }

    /**
     * @return string
     */
    public function getAccountName()
    {
        return $this->accountName;
    }

    /**
     * @param array $data
     * @return ConnectionResponse
     */
    private function setAccountName(array $data)
    {
        $this->ensureArrayKeyExists('accountName', $data);
        $this->ensureString($data['accountName']);
        $this->accountName = $data['accountName'];
        return $this;
    }

    /**
     * @return string
     */
    public function getProjectName()
    {
        return $this->projectName;
    }

    /**
     * @param array $data
     * @return ConnectionResponse
     */
    private function setProjectName(array $data)
    {
        $this->ensureArrayKeyExists('projectName', $data);
        $this->ensureString($data['projectName']);
        $this->projectName = $data['projectName'];
        return $this;
    }

    /**
     * @return array|Article[]
     */
    public function getArticles()
    {
        return $this->articles;
    }

    /**
     * @param array $data
     * @return ConnectionResponse
     */
    private function setArticles(array $data)
    {
        $this->ensureArrayKeyExists('articles', $data);
        $this->ensureArray($data['articles']);

        foreach ($data['articles'] as $articleData) {
            $this->articles[] = Article::fromState($articleData);
        }

        return $this;
    }

    /**
     * @return array|Category[]
     */
    public function getCategories()
    {
        return $this->categories;
    }

    /**
     * @param array $data
     * @return ConnectionResponse
     */
    private function setCategories(array $data)
    {
        $this->ensureArrayKeyExists('categories', $data);
        $this->ensureArray($data['categories']);

        foreach ($data['categories'] as $categoryData) {
            $this->categories[] = Category::fromState($categoryData);
        }

        return $this;
    }

    /**
     * @return WorkflowStatus[]
     */
    public function getStages()
    {
        return $this->stages;
    }

    /**
     * @param array $data
     * @return ConnectionResponse
     */
    private function setStages(array $data)
    {
        $this->ensureArrayKeyExists('stages', $data);
        $this->ensureArray($data['stages']);

        foreach ($data['stages'] as $stageData) {
            $this->stages[] = WorkflowStatus::fromState($stageData);
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getEditUrlBase()
    {
        return $this->editUrlBase;
    }

    /**
     * @param array $data
     * @return ConnectionResponse
     */
    private function setEditUrlBase(array $data)
    {
        $this->ensureArrayKeyExists('editUrlBase', $data);
        $this->ensureString($data['editUrlBase']);
        $this->editUrlBase = $data['editUrlBase'];
        return $this;
    }
}
