<?php

namespace EasyContent\WordPress\Source\Api\Entity;

final class WorkflowStatus extends Entity implements EntityInterface
{
    /** @var int $id */
    private $id;
    /** @var string $color */
    private $color;
    /** @var string $name */
    private $name;
    /** @var int $isCore */
    private $isCore;

    private function __construct() { }

    /**
     * @param array $data
     * @return self
     */
    public static function fromState(array $data)
    {
        return (new self)
            ->setId($data)
            ->setColor($data)
            ->setName($data)
            ->setIsCore($data);
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @param array $data
     * @return WorkflowStatus
     */
    private function setId(array $data)
    {
        $this->ensureArrayKeyExists('id', $data);
        $this->ensureInteger($data['id']);
        $this->id = $data['id'];
        return $this;
    }

    /**
     * @return string
     */
    public function getColor()
    {
        return $this->color;
    }

    /**
     * @param array $data
     * @return WorkflowStatus
     */
    private function setColor(array $data)
    {
        $this->ensureArrayKeyExists('color', $data);
        $this->ensureString($data['color']);
        $this->color = $data['color'];
        return $this;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param array $data
     * @return WorkflowStatus
     */
    private function setName(array $data)
    {
        $this->ensureArrayKeyExists('name', $data);
        $this->ensureString($data['name']);
        $this->name = $data['name'];
        return $this;
    }

    /**
     * @return int
     */
    public function getIsCore()
    {
        return $this->isCore;
    }

    /**
     * @param array $data
     * @return WorkflowStatus
     */
    private function setIsCore(array $data)
    {
        $this->ensureArrayKeyExists('is_core', $data);
        $this->ensureInteger($data['is_core']);
        $this->isCore = $data['is_core'];
        return $this;
    }

    /**
     * @param WorkflowStatus $item
     * @return bool
     */
    public function fullyEquals($item)
    {
        return $this->id === $item->id
            && $this->color === $item->color
            && $this->name === $item->name
            && $this->isCore === $item->isCore;
    }
}
