<?php

namespace EasyContent\WordPress\Source\Services;

use EasyContent\WordPress\Source\Api\Entity\EntityInterface;
use EasyContent\WordPress\Source\Entity\CacheCheckResult;
use EasyContent\WordPress\Source\Entity\LocalCache;

final class CacheUpdater
{
    /**
     * @param LocalCache $repository
     * @param array|EntityInterface[] $fetchedItems
     * @return void
     */
    public function updateCache(LocalCache $repository, array $fetchedItems)
    {
        $cached = $repository->all();
        $result = $this->calculate($cached, $fetchedItems);

        if ($result->getDelete()) {
            $repository->batchDelete($result->getDelete());
        }

        foreach ($result->getInsert() as $item) {
            $repository->insert($item);
        }

        foreach ($result->getCheckUpdate() as $id => $item) {
            $freshItem = $result->getFreshRowById($id);

            if (!$item->fullyEquals($freshItem)) {
                $repository->update($freshItem);
            }
        }
    }

    /**
     * @param array|EntityInterface[] $old
     * @param array|EntityInterface[] $new
     * @return CacheCheckResult
     */
    private function calculate(array $old, array $new)
    {
        $oldData = $this->index($old);
        $newData = $this->index($new);

        return new CacheCheckResult(
            array_diff_key($oldData, $newData),
            array_diff_key($newData, $oldData),
            array_intersect_key($oldData, $newData), // Stores values from first array
            $newData
        );
    }

    /**
     * @param array|EntityInterface[] $items
     * @return array|EntityInterface[]
     */
    private function index(array $items)
    {
        $result = [];

        foreach ($items as $entity) {
            $result[$entity->getId()] = $entity;
        }

        return $result;
    }
}
