<?php

namespace EasyContent\WordPress\Source\Services;

use EasyContent\WordPress\Source\Entity\PluginSettings;

final class CategoriesMapper
{
    /**
     * This method is used when importing article to WordPress.
     *
     * @param PluginSettings $settings
     * @param array|int[] $categories EasyContent topic's category ids array
     * @return array|int[] Ids of categories that new post must be in.
     */
    public static function wpFromEcCategoriesIds(PluginSettings $settings, array $categories)
    {
        if (!$categories) {
            return [];
        }

        $termIds = [];
        $flippedCategories = array_flip($categories);

        foreach ($settings->getMappings() as $termId => $easyContentCategoryId) {
            if (!isset($flippedCategories[$easyContentCategoryId])) {
                continue;
            }

            $termIds[] = $termId;
        }

        if (!$termIds) {
            $termIds = [$settings->getDefaultCategoryId()];
        }

        return array_unique($termIds);
    }

    /**
     * This method is used when pushing post to EasyContent.
     *
     * @param PluginSettings $settings
     * @param array|int[] $categoriesIds
     * @return array|int[]
     */
    public static function ecFromWpCategoriesIds(PluginSettings $settings, array $categoriesIds)
    {
        if (!$categoriesIds) {
            return [];
        }

        $ecCategoriesIds = [];
        $mappings = $settings->getMappings();

        foreach ($categoriesIds as $termId) {
            if ($mappings[$termId] <= 0) {
                continue;
            }

            $ecCategoriesIds[] = (int)$mappings[$termId];
        }

        return $ecCategoriesIds;
    }
}
