<?php

namespace EasyContent\WordPress\Source\Services;

use EasyContent\WordPress\Helpers\TimeHelper;
use EasyContent\WordPress\Plugin;
use EasyContent\WordPress\Source\Entity\CachedArticlesStorage;
use EasyContent\WordPress\Source\Entity\Post\PostStorage;
use EasyContent\WordPress\Source\Entity\WorkflowStatusStorage;
use EasyContent\WordPress\Source\Exception\CachedArticleNotFoundException;
use EasyContent\WordPress\Source\Exception\WorkflowStatusNotFoundException;

final class FrontendDataProvider
{
    private $posts;
    private $cachedArticles;
    private $statuses;

    public function __construct(
        PostStorage $posts,
        CachedArticlesStorage $cachedArticles,
        WorkflowStatusStorage $statuses
    ) {
        $this->posts = $posts;
        $this->cachedArticles = $cachedArticles;
        $this->statuses = $statuses;
    }

    /**
     * @param int $postId
     * @return array
     * @throws CachedArticleNotFoundException
     * @throws WorkflowStatusNotFoundException
     */
    public function getData($postId)
    {
        $post = $this->posts->getById($postId);

        if (!$post->getRelatedArticleId()) {
            throw new \DomainException('Post was not synchronized. Push first!');
        }

        $cachedArticle = $this->cachedArticles->getById($post->getRelatedArticleId());
        $currentStatus = $this->statuses->getById($cachedArticle->getCurrentWorkflowStatusId());

        $base = Plugin::getInstance()->getMisc()->editUrlBase;
        $editRemoteArticleUrl = $base ? "{$base}{$cachedArticle->getId()}" : '#';

        $syncTime = $post->getSyncTime() ? TimeHelper::getArticleListTableDateString($post->getSyncTime()) : '';

        return [
            'id'        => $cachedArticle->getId(), // Remote article ID. Used for endpoints generation
            'isSynced'  => true,
            'title'     => $cachedArticle->getTitle(),
            'topicName' => $cachedArticle->getTopicName(),
            'url'       => $editRemoteArticleUrl,
            'content'   => $post->getContent(), // This probably doesn't work. TODO: Fix this and related quickfixes
            'stage'     => [
                'id'    => $currentStatus->getId(),
                'name'  => $currentStatus->getName(),
                'color' => $currentStatus->getColor(),
            ],
            'lastUpdatedString' => $syncTime,
        ];
    }
}
