<?php

namespace EasyContent\WordPress\Source\UseCase\PublishArticle;

use EasyContent\WordPress\Source\Entity\CachedArticlesStorage;
use EasyContent\WordPress\Source\Entity\EasyContentArticlesStorage;
use EasyContent\WordPress\Source\Entity\PluginSettings;
use EasyContent\WordPress\Source\Entity\PluginStorage;
use EasyContent\WordPress\Source\Entity\Post;
use EasyContent\WordPress\Source\Exception\RemoteArticleNotFoundException;

/**
 * This class is used to handle EasyContent-to-WordPress request to publish article
 */
final class Handler
{
    private $remoteArticles;
    private $pluginSettings;
    private $cachedArticles;
    private $pluginStorage;

    public function __construct(
        EasyContentArticlesStorage $remoteArticles,
        PluginSettings $pluginSettings,
        CachedArticlesStorage $cachedArticles,
        PluginStorage $pluginStorage
    ) {
        $this->remoteArticles = $remoteArticles;
        $this->pluginSettings = $pluginSettings;
        $this->cachedArticles = $cachedArticles;
        $this->pluginStorage = $pluginStorage;
    }

    public function handle(Command $command)
    {
        try {
            $remoteArticle = $this->remoteArticles->getById($command->getRemoteArticleId());

            if ($remoteArticle->getRelatedPost()) {
                $remoteArticle->getRelatedPost()->update($remoteArticle, $this->pluginSettings, $command->getExtra());
            } else {
                $remoteArticle->setRelatedPost(
                    Post::create($this->pluginSettings, $remoteArticle, $command->getExtra())
                );
            }

            $remoteArticle->updateLocalCache();

            $this->pluginStorage->save($remoteArticle);
        } catch (RemoteArticleNotFoundException $e) {
            $this->cachedArticles->removeByIds([$command->getRemoteArticleId()->asInt()]);
        }
    }
}
