<?php

namespace EasyContent\WordPress\Source\UseCase\RefreshCachedData;

use EasyContent\WordPress\Source\Api\Entity\ImportedArticle;
use EasyContent\WordPress\Source\Entity\CachedArticle;
use EasyContent\WordPress\Source\Entity\CachedArticlesStorage;
use EasyContent\WordPress\Source\Entity\CategoriesStorage;
use EasyContent\WordPress\Source\Entity\EasyContentStorage;
use EasyContent\WordPress\Source\Entity\WorkflowStatusStorage;
use EasyContent\WordPress\Source\Services\CacheUpdater;

final class Handler
{
    private $remoteRepository;
    private $categories;
    private $articles;
    private $statuses;
    private $cacheUpdater;

    public function __construct(
        EasyContentStorage $remoteRepository,
        CategoriesStorage $categories,
        CachedArticlesStorage $articles,
        WorkflowStatusStorage $statuses
    ) {
        $this->remoteRepository = $remoteRepository;
        $this->categories = $categories;
        $this->articles = $articles;
        $this->statuses = $statuses;
        $this->cacheUpdater = new CacheUpdater;
    }

    public function handle()
    {
        $categories = $this->remoteRepository->getCategories();
        $articles = $this->remoteRepository->getArticles();
        $statuses = $this->remoteRepository->getStatuses();

        $this->cacheUpdater->updateCache($this->categories, $categories);
        $this->cacheUpdater->updateCache($this->articles, $this->convertArticles($articles));
        $this->cacheUpdater->updateCache($this->statuses, $statuses);
    }

    /**
     * @param array|ImportedArticle[] $articles
     * @return array|CachedArticle[]
     */
    private function convertArticles(array $articles)
    {
        return array_map(function(ImportedArticle $article) { return CachedArticle::fromImported($article); }, $articles);
    }
}
