<?php

namespace EasyContent\WordPress\Source\UseCase\SetNewStatus;

use EasyContent\WordPress\Source\Entity\Article\ArticleId;
use EasyContent\WordPress\Source\Entity\CachedArticlesStorage;
use EasyContent\WordPress\Source\Entity\EasyContentArticlesStorage;
use EasyContent\WordPress\Source\Entity\Post\PostStorage;
use EasyContent\WordPress\Source\Entity\WorkflowStatusStorage;
use EasyContent\WordPress\Source\Exception\RemoteArticleNotFoundException;

final class Handler
{
    private $remoteArticles;
    private $statuses;
    private $cachedArticles;
    private $posts;

    public function __construct(
        EasyContentArticlesStorage $remoteArticles,
        WorkflowStatusStorage $statuses,
        CachedArticlesStorage $cachedArticles,
        PostStorage $posts
    ) {
        $this->remoteArticles = $remoteArticles;
        $this->statuses = $statuses;
        $this->cachedArticles = $cachedArticles;
        $this->posts = $posts;
    }

    public function handle(Command $command)
    {
        if (!$this->statuses->existsById($command->getStatusId())) {
            $message = 'Status "%s" doesn\'t exist. Update cache ant try again or contact us.';
            throw new \DomainException(sprintf($message, $command->getStatusId()));
        }

        $post = $this->posts->getById($command->getPostId());

        if (!$post->getRelatedArticleId()) {
            throw new \DomainException('This post has no related article. Unable to change workflow status.');
        }

        try {
            $article = $this->remoteArticles->getById(new ArticleId($post->getRelatedArticleId()));

            if ($command->getStatusId() !== $article->getCurrentWorkflowStatusId()) {
                $article->setCurrentWorkflowStatusId($command->getStatusId());
                $this->remoteArticles->updateStatus($article);
                $article->updateLocalCache();
                $this->cachedArticles->update($article->getCached());
            }
        } catch (RemoteArticleNotFoundException $e) {
            $this->cachedArticles->removeByIds([$post->getRelatedArticleId()]);
        }
    }
}
