<?php

namespace EasyContent\WordPress\ValueObjects;

final class AttachmentPost
{
    /** @var int $id */
    private $id;

    /** @var string $localUrl Represents full URL path to the original attachment e.g. http://easycontent.wp/wp-content/uploads/2022/11/salmon-gb252d39fd_1920.jpg */
    private $localUrl;

    /**
     * @param int $id
     * @param string $localUrl
     */
    public function __construct($id, $localUrl)
    {
        $this->id = $id;
        $this->localUrl = $localUrl;
    }

    /**
     * @param array $data
     * @return null|AttachmentPost
     */
    public static function fromArray(array $data)
    {
        $attachmentId = (int)$data['ID'];
        $url = self::getAttachedFile($attachmentId, $data['meta_value']);

        if (!$url) {
            return null;
        }

        return new self($attachmentId, $url);
    }

    /**
     * This is custom implementation of built-in "get_attached_file" function
     *
     * @param int $attachmentId
     * @param null|string $file
     * @return null|string The file path to where the attached file should be, false otherwise.
     */
    private static function getAttachedFile($attachmentId, $file)
    {
        // If the file is relative, prepend upload dir.
        if ( $file && 0 !== strpos( $file, '/' ) && ! preg_match( '|^.:\\\|', $file ) ) {
            $uploads = wp_get_upload_dir();
            if ( false === $uploads['error'] ) {
                $file = $uploads['basedir'] . "/$file";
            }
        }

        return apply_filters('get_attached_file', $file, $attachmentId);
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string
     */
    public function getLocalUrl()
    {
        return $this->localUrl;
    }
}
